.cv-property {
    margin-bottom: 10px;
    display: flex;
}

.cv-property-name {
    font-weight: bold;
    width: 200px; /* Adjust this value as needed */
    text-align: right;
    margin-right: 10px;
}

.cv-property-value {
    color: #666;
}
