# Generated by Django 4.2 on 2026-02-20 10:26

from django.db import migrations
import django_fsm


class Migration(migrations.Migration):
    dependencies = [
        ("app", "0004_campaign"),
    ]

    operations = [
        migrations.AlterField(
            model_name="campaign",
            name="state",
            field=django_fsm.FSMField(
                choices=[
                    ("new", "New"),
                    ("active", "Created"),
                    ("success", "Error"),
                    ("canceled", "Assigned"),
                    ("error", "Error"),
                ],
                default="new",
                max_length=50,
            ),
        ),
    ]
