<a {% if cv_list_action_method == "get" %}
    href="{{ cv_url }}"
{% elif cv_list_action_method == "post" %}
    href="javascript:{}"
    onclick="return cv_list_action_form_submit('{{ cv_oid }}')"
{% endif %}
    class="btn btn-outline-{% if cv_access is True %}primary{% else %}secondary{% endif %} btn-xs{% if cv_access is not True %} disabled{% endif %}"
    role="button"
    title="{{ cv_action_label }}"
    cv-key="{{ cv_key }}">
    <i class="{{ cv_icon_action }}"></i>
</a>
