{% extends cv_extends %}

{% block cv_filter %}
    {% include "crud_views/view_list_table.filter.html" %}
{% endblock cv_filter %}

{% block cv_content %}
    {% include "crud_views/view_list_table.content.html" %}
{% endblock cv_content %}
