<a href="#"
   class="btn btn-outline-primary btn-lg"
   id="cv-filter-toggle"
   role="button"
   title="{{ cv_action_label }}"
   cv-key="{{ cv_key }}">
    <i class="{{ cv_icon_action }}"></i>
</a>
