{% extends "app/base.html" %}

{% block content %}

    <div class="bg-body-tertiary p-3 rounded">
        <h1>Django ViewSet Example</h1>
        <h3>Bootstrap5 Theme</h3>
        <p class="lead">
            This example demonstrates the `viewset_bootstrap` theme.
        </p>
        <a class="btn btn-lg btn-primary" href="/docs/5.3/components/navbar/" role="button">
            View navbar docs &raquo;
        </a>
    </div>

{% endblock content %}
