{% extends "app/base.html" %}

{% block body %}
    <h1>LOGIN</h1>

    <form method="post">
        {% csrf_token %}
        {{ form }}
        <button type="submit">Log In</button>
    </form>


{% endblock body %}