# Generated by Django 4.2 on 2026-02-20 08:08

import crud_views_workflow.lib.mixins
from django.db import migrations, models
import django_fsm


class Migration(migrations.Migration):
    dependencies = [
        ("app", "0003_alter_polyanswer_options_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="Campaign",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=128)),
                (
                    "state",
                    django_fsm.FSMField(
                        choices=[
                            ("new", "New"),
                            ("active", "Created"),
                            ("success", "Error"),
                            ("canceled", "Assigned"),
                        ],
                        default="new",
                        max_length=50,
                    ),
                ),
            ],
            bases=(crud_views_workflow.lib.mixins.WorkflowModelMixin, models.Model),
        ),
    ]
