{% load crud_views %}

<div type="{{ widget.type }}" name="{{ widget.name }}"{% include "django/forms/widgets/attrs.html" %}>
    <i class="fa-solid fa-angles-right"></i>
    {% if widget.value|cv_is_true %}
        <i class="fa-regular fa-square-check"></i>
    {% elif widget.value|cv_is_false %}
        <i class="fa-regular fa-square"></i>
    {% else %}
        -
    {% endif %}
</div>
