{% extends "app/base.html" %}

{% block content %}

<div class="bg-body-tertiary p-3 rounded">
    <h1>Django CRUD Views Example</h1>
    <h3>Bootstrap5 Theme</h3>
    <p class="lead">
        This example demonstrates the default Bootstrap 5 theme included in <b>crud_views</b>.
    </p>
    <a class="btn btn-lg btn-primary" href="https://django-crud-views.readthedocs.io/en/latest" role="button">
        Django CRUD View Documentation &raquo;
    </a>
</div>

{% endblock content %}
