# Generated by Django 5.2.11 on 2026-02-20 17:08

import django_fsm
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("app", "0005_alter_campaign_state"),
    ]

    operations = [
        migrations.AlterField(
            model_name="campaign",
            name="state",
            field=django_fsm.FSMField(
                choices=[
                    ("new", "New"),
                    ("active", "Active"),
                    ("success", "Success"),
                    ("canceled", "Cancelled"),
                    ("error", "Error"),
                ],
                default="new",
                max_length=50,
            ),
        ),
    ]
