.cv-col-id {
    width: 5% !important;
}

.cv-col-5 {
    width: 5% !important;
}

.cv-col-10 {
    width: 10% !important;
}

.cv-col-15 {
    width: 15% !important;
}

.cv-col-20 {
    width: 20% !important;
}

.cv-col-25 {
    width: 25% !important;
}

.cv-col-30 {
    width: 30% !important;
}

.cv-col-35 {
    width: 35% !important;
}

.cv-col-40 {
    width: 40% !important;
}

.cv-col-45 {
    width: 45% !important;
}

.cv-col-50 {
    width: 50% !important;
}

.cv-col-55 {
    width: 55% !important;
}

.cv-col-60 {
    width: 60% !important;
}

.cv-col-65 {
    width: 65% !important;
}

.cv-col-70 {
    width: 70% !important;
}

.cv-col-75 {
    width: 75% !important;
}

.cv-col-80 {
    width: 80% !important;
}

.cv-col-90 {
    width: 90% !important;
}

.cv-col-95 {
    width: 95% !important;
}

.cv-col-100 {
    width: 100% !important;
}
