{% if cv_key %}
    {% if cv_access is True %}
        <a href="{{ cv_url }}" class="btn btn-outline-primary {% if cv_is_active %}active{% endif %} btn-lg"
           role="button"
           title="{{ cv_action_label }}"
           cv-key="{{ cv_key }}">
            <i class="{{ cv_icon_action }}"></i>
        </a>
    {% else %}
        <a href="{{ cv_url }}"
           class="btn btn-outline-secondary btn-lg disabled"
           role="button"
           cv-key="{{ cv_key }}"
           title="{{ cv_action_label }}">
            <i class="{{ cv_icon_action }}"></i>
        </a>
    {% endif %}
{% endif %}
