# Generated by Django 5.1.4 on 2024-12-27 14:59

import django.db.models.deletion
import uuid
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("app", "0001_initial"),
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Poly",
            fields=[
                ("id", models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ("shared", models.CharField(max_length=100)),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_%(app_label)s.%(class)s_set+",
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "base_manager_name": "objects",
            },
        ),
        migrations.CreateModel(
            name="PolyOne",
            fields=[
                (
                    "poly_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="app.poly",
                    ),
                ),
                ("one", models.CharField(max_length=100)),
            ],
            options={
                "abstract": False,
                "base_manager_name": "objects",
            },
            bases=("app.poly",),
        ),
        migrations.CreateModel(
            name="PolyTwo",
            fields=[
                (
                    "poly_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="app.poly",
                    ),
                ),
                ("two", models.CharField(max_length=100)),
            ],
            options={
                "abstract": False,
                "base_manager_name": "objects",
            },
            bases=("app.poly",),
        ),
    ]
