# Generated by Django 5.2.11 on 2026-02-20 07:40

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("app", "0002_question_questionchoice_questionchoicetag_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="polyanswer",
            options={},
        ),
        migrations.AlterModelOptions(
            name="polyanswernumber",
            options={},
        ),
        migrations.AlterModelOptions(
            name="polyanswertext",
            options={},
        ),
        migrations.AlterModelOptions(
            name="polyone",
            options={},
        ),
        migrations.AlterModelOptions(
            name="polythree",
            options={},
        ),
        migrations.AlterModelOptions(
            name="polytwo",
            options={},
        ),
    ]
